@@ -1261,6 +1261,7 @@ class Backtest:
         strategy: Strategy = self._strategy(broker, data, kwargs)
 
         def prepare_strategy():
+            data._set_length(len(self._data))
             strategy.init()
             data._update()  # Strategy.init might have changed/added to data.df
             # Indicators used in Strategy.next()
@@ -1352,11 +1353,16 @@ class Backtest:
         self._data = datacopy
 
         # Correlation
-        outSampleLen = len(outsample_run["_equity_curve"])
-        samples_corr = np.corrcoef(
-            outsample_run["_equity_curve"]["Equity"],
-            insample_optimization[0]["_equity_curve"]["Equity"][-outSampleLen:],
-        )[1, 0]
+        insample_change = (
+            insample_optimization[0]["_equity_curve"]["Equity"].pct_change().fillna(0)
+        )
+        outsample_change = (
+            outsample_run["_equity_curve"]["Equity"].pct_change().fillna(0)
+        )
+        outSampleLen = len(outsample_change)
+        samples_corr = np.corrcoef(outsample_change, insample_change[-outSampleLen:])[
+            1, 0
+        ]
 
         return insample_optimization, outsample_run, samples_corr
 
@@ -1377,6 +1383,7 @@ class Backtest:
         strategy: Strategy = self._strategy(broker, data, dict([]))
 
         def prepare_strategy():
+            data._set_length(len(self._data))
             strategy.init()
             data._update()  # Strategy.init might have changed/added to data.df
             # Indicators used in Strategy.next()
@@ -1483,6 +1490,7 @@ class Backtest:
         strategy: Strategy = self._strategy(broker, data, dict([]))
 
         def prepare_strategy():
+            data._set_length(len(self._data))
             strategy.init()
             data._update()  # Strategy.init might have changed/added to data.df
             # Indicators used in Strategy.next()
